package ink.wgink.service.department.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONArray;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SaveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.bos.DepartmentBO;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.department.DepartmentDTO;
import ink.wgink.pojo.dtos.department.DepartmentSimpleDTO;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.pojo.result.UploadExcelResultDTO;
import ink.wgink.service.department.dao.IDepartmentDao;
import ink.wgink.service.department.listener.excel.DepartmentExcel;
import ink.wgink.service.department.listener.excel.DepartmentExcelError;
import ink.wgink.service.department.listener.excel.DepartmentExcelListener;
import ink.wgink.service.department.pojo.vos.DepartmentVO;
import ink.wgink.service.department.pojo.vos.MergeDepartmentInfoVO;
import ink.wgink.service.department.pojo.vos.MergeNewDepartmentInfoVO;
import ink.wgink.service.department.pojo.vos.SplitDepartmentVO;
import ink.wgink.service.department.service.IDepartmentService;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.map.HashMapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: DepartmentServiceImpl
 * @Description: 组织部门
 * @Author: wanggeng
 * @Date: 2021/1/27 3:53 下午
 * @Version: 1.0
 */
@Service
public class DepartmentServiceImpl extends DefaultBaseService implements IDepartmentService {

    @Autowired
    private IDepartmentDao departmentDao;

    @Override
    public void save(DepartmentVO departmentVO) {
        saveReturnId(departmentVO);
    }

    @Override
    public String saveReturnId(DepartmentVO departmentVO) {
        String parentCode = null;
        String departmentParentId = departmentVO.getDepartmentParentId();
        if (!ISystemConstant.TREE_ROOT_ID.equals(departmentParentId)) {
            DepartmentDTO departmentDTO = get(departmentParentId);
            parentCode = departmentDTO.getDepartmentCode();
        } else {
            if (!isAdmin()) {
                throw new SaveException("非超级管理员无法在根节点添加组织");
            }
        }
        // 同级，同名组织不能添加
        if (isDepartmentExist(null, departmentVO.getDepartmentParentId(), departmentVO.getDepartmentName())) {
            throw new SaveException("同级组织名称不能相同");
        }
        String departmentId = UUIDUtil.getUUID();
        String departmentCode = getCode(parentCode, departmentParentId);
        Map<String, Object> params = HashMapUtil.beanToMap(departmentVO);
        setArea(departmentVO, params);
        params.put("departmentCode", departmentCode);
        params.put("departmentId", departmentId);
        setSaveInfo(params);
        departmentDao.save(params);
        return departmentId;
    }

    @Override
    public void remove(List<String> ids) {
        // 清空组织所在人员组织信息
        for (String departmentId : ids) {
            userService.deleteDepartmentUserByDepartmentId(departmentId);
        }
        Map<String, Object> params = getHashMap(2);
        params.put("departmentIds", ids);
        setUpdateInfo(params);
        departmentDao.remove(params);
    }

    @Override
    public void update(String departmentId, DepartmentVO departmentVO) {
        if (isDepartmentExist(departmentId, departmentVO.getDepartmentParentId(), departmentVO.getDepartmentName())) {
            throw new SaveException("同级组织名称不能相同");
        }
        Map<String, Object> params = HashMapUtil.beanToMap(departmentVO);
        setArea(departmentVO, params);
        params.put("departmentId", departmentId);
        setUpdateInfo(params);
        departmentDao.update(params);
    }

    @Override
    public void updateMergeNew(MergeNewDepartmentInfoVO mergeNewDepartmentInfoVO) {
        String departmentId = saveReturnId(mergeNewDepartmentInfoVO);
        String[] mergeDepartmentIdArray = mergeNewDepartmentInfoVO.getMergeDepartmentIds().split("_");
        mergeByDepartmentId(departmentId, mergeNewDepartmentInfoVO.getDepartmentName(), mergeDepartmentIdArray);
    }

    @Override
    public void updateMerge(String departmentId, MergeDepartmentInfoVO mergeDepartmentInfoVO) {
        DepartmentSimpleDTO departmentSimpleDTO = getSimpleById(departmentId);
        if (departmentSimpleDTO == null) {
            throw new SearchException("部门不存在");
        }
        String[] mergeDepartmentIdArray = mergeDepartmentInfoVO.getMergeDepartmentIds().split("_");
        mergeDepartmentByDepartmentId(departmentId, departmentSimpleDTO.getDepartmentName(), mergeDepartmentIdArray);
    }

    @Override
    public void updateSplit(String departmentId, SplitDepartmentVO splitDepartmentVO) {

    }

    @Override
    public void resetCode() {
        Map<String, Object> params = getHashMap(2);
        resetCode("0", "", params);
    }


    @Override
    public List<DepartmentDTO> list(Map<String, Object> params) {
        return departmentDao.list(params);
    }

    @Override
    public SuccessResultList<List<DepartmentDTO>> listPage(ListPage page) {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<DepartmentDTO> departmentDTOs = list(page.getParams());
        PageInfo<DepartmentDTO> pageInfo = new PageInfo<>(departmentDTOs);
        return new SuccessResultList<>(departmentDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    @Override
    public SuccessResultList<List<DepartmentDTO>> listPageByParentId(String departmentParentId, ListPage page) {
        if (StringUtils.equals(ISystemConstant.TREE_ROOT_ID, departmentParentId)) {
            List<String> listBaseDepartmentIds = listBaseDepartmentIds();
            if (!Objects.isNull(listBaseDepartmentIds)) {
                page.getParams().put("departmentIds", listBaseDepartmentIds);
            } else {
                page.getParams().put("departmentParentId", departmentParentId);
            }
            return listPage(page);
        }
        page.getParams().put("departmentParentId", departmentParentId);
        return listPage(page);
    }

    @Override
    public DepartmentDTO get(String departmentId) {
        Map<String, Object> params = getHashMap(1);
        params.put("departmentId", departmentId);
        return departmentDao.get(params);
    }

    @Override
    public List<DepartmentDTO> listAll(Map<String, Object> params) {
        List<DepartmentDTO> departmentDTOs = list(params);
        listSub(departmentDTOs, params);
        return departmentDTOs;
    }

    @Override
    public List<ZTreeDTO> listZTree(Map<String, Object> params) {
        List<ZTreeDTO> zTreeDepartment = departmentDao.listZTree(params);
        List<ZTreeDTO> zTreeDTOs = Arrays.asList(new ZTreeDTO[zTreeDepartment.size()]);
        Collections.copy(zTreeDTOs, zTreeDepartment);
        for (ZTreeDTO zTreeDTO : zTreeDTOs) {
            Integer subCount = departmentDao.countByParentId(zTreeDTO.getId());
            setZTreeInfo(zTreeDTO, subCount);
        }
        return zTreeDTOs;
    }

    @Override
    public List<ZTreeDTO> listZTree(String departmentParentId, Map<String, Object> params) {
        if (StringUtils.equals(ISystemConstant.TREE_ROOT_ID, departmentParentId)) {
            List<String> listBaseDepartmentIds = listBaseDepartmentIds();
            if (listBaseDepartmentIds != null && !listBaseDepartmentIds.isEmpty()) {
                params.put("departmentIds", listBaseDepartmentIds);
                return listZTree(params);
            }
        }
        params.put("departmentParentId", departmentParentId);
        return listZTree(params);
    }

    @Override
    public List<String> listId(Map<String, Object> params) {
        List<String> departmentIds = new ArrayList<>();
        List<DepartmentDTO> departmentDTOs = list(params);
        for (DepartmentDTO departmentDTO : departmentDTOs) {
            departmentIds.add(departmentDTO.getDepartmentId());
        }
        return departmentIds;
    }

    @Override
    public List<String> listId(List<DepartmentBO> departmentBOs) {
        List<String> departmentIds = new ArrayList<>();
        if (Objects.isNull(departmentBOs) || departmentBOs.isEmpty()) {
            return departmentIds;
        }
        List<DepartmentBO> departmentList = new ArrayList<>(departmentBOs);
        List<DepartmentBO> departmentTempList = new ArrayList<>(departmentBOs);
        for (int i = 0; i < departmentList.size(); i++) {
            DepartmentBO departmentBO = null;
            // 找出CODE
            for (DepartmentBO department : departmentList) {
                DepartmentBO higherDepartmentBO = getHigherLevel(departmentList.get(i), department);
                if (higherDepartmentBO != null) {
                    departmentBO = higherDepartmentBO;
                }
            }
            departmentIds.add(departmentBO.getDepartmentId());
            // 删除包含关系的内容
            for (int j = 0; j < departmentList.size(); j++) {
                if (StringUtils.startsWith(departmentList.get(j).getDepartmentCode(), departmentBO.getDepartmentCode())) {
                    departmentList.remove(j);
                    j--;
                }
            }
            for (int j = 0; j < departmentTempList.size(); j++) {
                if (StringUtils.startsWith(departmentTempList.get(j).getDepartmentCode(), departmentBO.getDepartmentCode())) {
                    departmentTempList.remove(j);
                    j--;
                }
            }
            i = -1;
        }
        return departmentIds;
    }

    @Override
    public DepartmentBO getHigherLevel(DepartmentBO departmentBO1, DepartmentBO departmentBO2) {
        if (StringUtils.startsWith(departmentBO1.getDepartmentCode(), departmentBO2.getDepartmentCode())) {
            return departmentBO2;
        } else if (StringUtils.startsWith(departmentBO2.getDepartmentCode(), departmentBO1.getDepartmentCode())) {
            return departmentBO1;
        }
        return null;
    }

    @Override
    public UploadExcelResultDTO uploadExcel(MultipartFile excel) {
        // 错误列表
        List<DepartmentExcelError> departmentExcelErrors = new ArrayList<>(0);
        long startTime = System.currentTimeMillis();
        EasyExcel.read(excel.getInputStream(), DepartmentExcel.class, new DepartmentExcelListener() {
            @Override
            public void listDepartmentExcel(List<DepartmentExcel> departmentExcels) throws Exception {
                JSONArray listTreeDepartmentExcelJsonArray = listTreeDepartmentExcelJsonArray(0, departmentExcels, departmentExcelErrors);
                saveExcel("0", null, listTreeDepartmentExcelJsonArray, departmentExcelErrors);
            }
        }).headRowNumber(2).sheet().doRead();
        long endTime = System.currentTimeMillis();
        String excelFileId = null;
        if (departmentExcelErrors.size() > 0) {
            excelFileId = new AbstractErrorExcelHandler<DepartmentExcelError>(fileService) {
                @Override
                public List<List<String>> excelHeaderNames() {
                    return simpleExcelHeader(new String[]{"ID", "组织名称", "上级ID", "组织说明", "组织电话", "类型", "错误原因"});
                }
            }.saveErrorExcel(departmentExcelErrors);
        }
        return new UploadExcelResultDTO(departmentExcelErrors.size(), endTime - startTime, excelFileId);
    }

    @Override
    public Integer count(Map<String, Object> params) {
        return null;
    }

    @Override
    public DepartmentSimpleDTO getSimpleById(String departmentId) {
        return null;
    }

    @Override
    public List<DepartmentSimpleDTO> listSimple(Map<String, Object> params) {
        return null;
    }

    @Override
    public List<DepartmentSimpleDTO> listSimpleByParentId(String departmentParentId) {
        return null;
    }

    /**
     * 设置组织组织地区
     *
     * @param departmentVO
     * @param params
     */
    private void setArea(DepartmentVO departmentVO, Map<String, Object> params) {
        if (departmentVO.getDepartmentArea1Id() != null) {
            DataAreaDTO dataAreaDTO = dataAreaService.getAreaById(departmentVO.getDepartmentArea1Id().toString());
            params.put("departmentArea1Code", dataAreaDTO.getAreaCode());
            params.put("departmentArea1Name", dataAreaDTO.getAreaName());
        } else {
            params.put("departmentArea1Code", "");
            params.put("departmentArea1Name", "");
        }
        if (departmentVO.getDepartmentArea2Id() != null) {
            DataAreaDTO dataAreaDTO = dataAreaService.getAreaById(departmentVO.getDepartmentArea2Id().toString());
            params.put("departmentArea2Code", dataAreaDTO.getAreaCode());
            params.put("departmentArea2Name", dataAreaDTO.getAreaName());
        } else {
            params.put("departmentArea2Code", "");
            params.put("departmentArea2Name", "");
        }
        if (departmentVO.getDepartmentArea3Id() != null) {
            DataAreaDTO dataAreaDTO = dataAreaService.getAreaById(departmentVO.getDepartmentArea3Id().toString());
            params.put("departmentArea3Code", dataAreaDTO.getAreaCode());
            params.put("departmentArea3Name", dataAreaDTO.getAreaName());
        } else {
            params.put("departmentArea3Code", "");
            params.put("departmentArea3Name", "");
        }
        if (departmentVO.getDepartmentArea4Id() != null) {
            DataAreaDTO dataAreaDTO = dataAreaService.getAreaById(departmentVO.getDepartmentArea4Id().toString());
            params.put("departmentArea4Code", dataAreaDTO.getAreaCode());
            params.put("departmentArea4Name", dataAreaDTO.getAreaName());
        } else {
            params.put("departmentArea4Code", "");
            params.put("departmentArea4Name", "");
        }
        if (departmentVO.getDepartmentArea5Id() != null) {
            DataAreaDTO dataAreaDTO = dataAreaService.getAreaById(departmentVO.getDepartmentArea5Id().toString());
            params.put("departmentArea5Code", dataAreaDTO.getAreaCode());
            params.put("departmentArea5Name", dataAreaDTO.getAreaName());
        } else {
            params.put("departmentArea5Code", "");
            params.put("departmentArea5Name", "");
        }
    }

    /**
     * 重置编码
     *
     * @param departmentParentId
     * @param departmentParentCode
     * @param params
     */
    private void resetCode(String departmentParentId, String departmentParentCode, Map<String, Object> params) {
        params.clear();
        params.put("departmentParentId", departmentParentId);
        List<DepartmentDTO> departmentDTOs = list(params);
        for (int i = 0; i < departmentDTOs.size(); i++) {
            params.clear();
            int index = i + 1;
            DepartmentDTO departmentDTO = departmentDTOs.get(i);
            String departmentCode = String.format("%s%04d", departmentParentCode, index);
            params.put("departmentId", departmentDTO.getDepartmentId());
            params.put("departmentCode", departmentCode);
            departmentDao.updateCode(params);
            resetCode(departmentDTO.getDepartmentId(), departmentCode, params);
        }
    }

    /**
     * 递归查询子部门
     *
     * @param departmentDTOs
     * @param params
     */
    private void listSub(List<DepartmentDTO> departmentDTOs, Map<String, Object> params) throws SearchException {
        for (DepartmentDTO departmentDTO : departmentDTOs) {
            params.put("departmentParentId", departmentDTO.getDepartmentId());
            List<DepartmentDTO> subDepartmentDTOs = list(params);
            departmentDTO.setSubDepartments(subDepartmentDTOs);
            listSub(subDepartmentDTOs, params);
        }
    }

    /**
     * 合并组织到指定组织
     *
     * @param departmentId
     * @param mergeDepartmentIdArray
     */
    private void mergeDepartmentByDepartmentId(String departmentId, String departmentName, String[] mergeDepartmentIdArray) {
        DepartmentDTO departmentDTO = get(departmentId);
        if (departmentDTO == null) {
            throw new SearchException("并入部门不存在");
        }
        List<String> departmentUserIds = new ArrayList<>();
        for (String mergeDepartmentId : mergeDepartmentIdArray) {
            DepartmentDTO mergeDepartmentDTO = get(mergeDepartmentId);
            if (mergeDepartmentDTO == null) {
                throw new SearchException("合并部门不存在");
            }
            LOG.debug("新增合并日志");
            saveDepartmentAdjustment(mergeDepartmentId, mergeDepartmentDTO.getDepartmentName(), departmentId, departmentDTO.getDepartmentName(), DepartmentAdjustmentTypeEnum.MERGE);
            LOG.debug("删除需要合并的组织人员");
            List<String> oldDepartmentUserIds = removeDepartmentByDepartmentIdReturnDeletedDepartmentUserIds(mergeDepartmentId, mergeDepartmentDTO.getDepartmentName());
            departmentUserIds.addAll(oldDepartmentUserIds);
        }
        LOG.debug("将合并组织的人员添加到新组织");
        userService.saveDepartmentUserIdByDepartmentId(departmentId, departmentName, departmentUserIds);
    }

    /**
     * 组织是否存在
     *
     * @param excludeDepartmentId
     * @param departmentParentId
     * @param departmentName
     * @return
     */
    private boolean isDepartmentExist(String excludeDepartmentId, String departmentParentId, String departmentName) {
        Map<String, Object> params = getHashMap(3);
        params.put("excludeDepartmentId", excludeDepartmentId);
        params.put("departmentParentId", departmentParentId);
        params.put("departmentName", departmentName.trim());
        DepartmentDTO departmentDTO = departmentDao.get(params);
        if (departmentDTO == null) {
            return false;
        }
        return false;
    }

    /**
     * 获取code
     *
     * @param parentCode
     * @param parentId
     * @return
     */
    private String getCode(String parentCode, String parentId) {
        DepartmentDTO departmentDTO = departmentDao.getLastByParentId(parentId);
        String code = departmentDTO != null ? departmentDTO.getDepartmentCode() : "0000";
        return super.getNewCode(code, parentCode);
    }
}
